@@ -1,3 +1,6 @@
+#The functions here are not worth to convert to C++ based on internal experiments
+#except for rpoisgamma, which was converted as rpoisgamma_rcpp, but given its complexity,
+#the R version is left as well (but it's 5-6x slower)
 
 # Global variables for CRAN check -----------------------------------------
 
@@ -247,20 +250,20 @@ qbeta_mse <- function(q,mean_v,se) {
 #' rgamma_mse(n=1,mean_v=0.8,se=0.2)
 #'
 
-rgamma_mse <- function(n=1,mean_v,se,seed=NULL) {
-  out <- NULL
-
-  if(!is.null(seed)){
-    set.seed(seed)
+rgamma_mse <- function(n=1, mean_v, se, seed=NULL) {
+  if (!is.null(seed)) set.seed(seed)
+  len <- length(mean_v)
+  scale <- se^2 / mean_v
+  shape <- mean_v / scale
+  out <- stats::rgamma(len * n, shape, scale = scale)
+  det <- (se == 0 | mean_v == 0)
+  det[is.na(det)] <- FALSE
+  if (any(det)) {
+    for (i in which(det)) {
+      out[seq.int(i, len * n, by = len)] <- mean_v[i]
+    }
   }
-  
-    bool_se <- se==0 | mean_v==0
-    scale <- se^2 / mean_v
-    shape <- mean_v / scale
-    out <- rgamma(length(mean_v)*n,shape,scale=scale)
-    out[bool_se] <- mean_v[bool_se]
-
-  return(out)
+  out
 }
 
 # Quantile gamma distribution - accepts vectors--------
@@ -281,24 +284,19 @@ rgamma_mse <- function(n=1,mean_v,se,seed=NULL) {
 #' @examples
 #' qgamma_mse(q=0.5,mean_v=0.8,se=0.2)
 #'
-
-qgamma_mse <- function(q=1,mean_v,se,seed=NULL) {
-  out <- NULL
-  
-  if(!is.null(seed)){
-    set.seed(seed)
-  }
-  
-  bool_se <- se==0 | mean_v==0
+qgamma_mse <- function(q=1, mean_v, se, seed=NULL) {
+  if (!is.null(seed)) set.seed(seed)
+  len <- length(mean_v)
   scale <- se^2 / mean_v
   shape <- mean_v / scale
-  out <- qgamma(q,shape,scale=scale)
-  out[bool_se] <- mean_v[bool_se]
-  
-  return(out)
+  out <- stats::qgamma(q, shape, scale = scale)
+  det <- (se == 0 | mean_v == 0)
+  det[is.na(det)] <- FALSE
+  out[det] <- mean_v[det]
+  out
 }
 
-#' Draw from a Conditional Gompertz distribution (lower and upper bound)
+#' Draw from a conditional Gompertz distribution (lower and upper bound)
 #'
 #' @param n The number of observations to be drawn
 #' @param shape The shape parameter of the Gompertz distribution, defined as in the coef() output on a flexsurvreg object
@@ -354,29 +352,6 @@ rcond_gompertz <- function(n=1,shape,rate,lower_bound=0,seed=NULL){
 }
 
 
-#' Quantile function for conditional Gompertz distribution (lower bound only)
-#'
-#' @param rnd Vector of quantiles
-#' @param shape The shape parameter of the Gompertz distribution, defined as in the coef() output on a flexsurvreg object
-#' @param rate The rate parameter of the Gompertz distribution, defined as in the coef() output on a flexsurvreg object
-#' @param lower_bound The lower bound of the conditional distribution
-#'
-#' @return Estimate(s) from the conditional Gompertz distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_gompertz(rnd=0.5,shape=0.05,rate=0.01,lower_bound = 50)
-
-qcond_gompertz <- function(rnd=0.5,shape,rate,lower_bound=0){
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
-  }
-  
-  out <- (1/shape)*log(1- ((shape/rate)*log(1-rnd)/exp(shape*lower_bound)))
-  
-  return(out)
-}
 
 #' Survival Probaility function for conditional Gompertz distribution (lower bound only)
 #'
@@ -640,432 +615,130 @@ cond_dirichlet <- function(alpha, i, xi, full_output = FALSE) {
   }
 }
 
-#' Conditional quantile function for exponential distribution 
+#' Creates a discrete resource from an external pointer
 #'
-#' @param rnd Vector of quantiles
-#' @param rate The rate parameter
+#' @param xptr external pointer from resource_discrete()$.ptr
 #' 
-#' Note taht the conditional quantile for an exponential is independent of time due to constant hazard
-#'
-#' @return Estimate(s) from the conditional exponential distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_exp(rnd = 0.5,rate = 3)
-
-qcond_exp <- function(rnd = 0.5, rate) {
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
+#' @return new cloned discrete resource
+#' 
+#' @keywords internal
+.resource_discrete_from_xptr <- function(xptr) {
+  env <- new.env(parent = emptyenv())
+  env$.ptr <- xptr
+  
+  env$size                 <- function()        discrete_resource_size_cpp(env$.ptr)
+  env$queue_size           <- function()        discrete_resource_queue_size_cpp(env$.ptr)
+  env$n_free               <- function()        discrete_resource_n_free_cpp(env$.ptr)
+  env$patients_using       <- function()        discrete_resource_patients_using_cpp(env$.ptr)
+  env$patients_using_times <- function()        discrete_resource_patients_using_times_cpp(env$.ptr)
+  env$queue_start_times    <- function()        discrete_resource_queue_start_times_cpp(env$.ptr)
+  env$queue_priorities     <- function()        discrete_resource_queue_priorities_cpp(env$.ptr)
+  
+  env$queue_info <- function(n = NULL) {
+    if (is.null(n)) n <- env$queue_size()
+    if (n <= 0) return(data.frame(patient_id = integer(0), priority = integer(0), start_time = numeric(0)))
+    ids  <- env$next_patient_in_line(n)
+    prio <- env$queue_priorities()[seq_len(length(ids))]
+    st   <- env$queue_start_times()[seq_len(length(ids))]
+    data.frame(patient_id = ids, priority = prio, start_time = st, stringsAsFactors = FALSE)
   }
   
-  -log(1-rnd)/rate
-}
-
-
-#' Conditional quantile function for weibull distribution 
-#'
-#' @param rnd Vector of quantiles
-#' @param shape The shape parameter as in R stats package weibull
-#' @param scale The scale parameter as in R stats package weibull
-#' @param lower_bound The lower bound to be used (current time)
-#'
-#' @return Estimate(s) from the conditional weibull distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_weibull(rnd = 0.5,shape = 3,scale = 66.66,lower_bound = 50)
-
-qcond_weibull <- function(rnd = 0.5, shape, scale, lower_bound=0) {
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
+  env$is_patient_in_queue <- function(patient_id) {
+    if (!is.numeric(patient_id) || length(patient_id) != 1) stop("patient_id must be a single number")
+    discrete_resource_is_patient_in_queue_cpp(env$.ptr, as.integer(patient_id))
+  }
+  env$is_patient_using <- function(patient_id) {
+    if (!is.numeric(patient_id) || length(patient_id) != 1) stop("patient_id must be a single number")
+    discrete_resource_is_patient_using_cpp(env$.ptr, as.integer(patient_id))
   }
   
-  ((lower_bound/scale)^shape - log(1-rnd))^(1/shape)*scale - lower_bound
-}
-
-#' Conditional quantile function for WeibullPH (flexsurv)
-#'
-#' @param rnd Vector of quantiles (between 0 and 1)
-#' @param shape Shape parameter of WeibullPH
-#' @param scale Scale (rate) parameter of WeibullPH (i.e., as in hazard = scale * t^(shape - 1))
-#' @param lower_bound Lower bound (current time)
-#'
-#' @return Estimate(s) from the conditional weibullPH distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_weibullPH(rnd = 0.5, shape = 2, scale = 0.01, lower_bound = 5)
-qcond_weibullPH <- function(rnd = 0.5, shape, scale, lower_bound = 0) {
-  if (any(rnd < 0 | rnd > 1)) {
-    stop("rnd must be between 0 and 1")
-  }
-  
-  base <- lower_bound^shape - log(1 - rnd) / scale
-  base[base < 0] <- NA  # to avoid complex numbers due to numerical issues
-  
-  residual <- base^(1 / shape) - lower_bound
-  return(residual)
-}
-
-
-#' Conditional quantile function for loglogistic distribution 
-#'
-#' @param rnd Vector of quantiles
-#' @param shape The shape parameter
-#' @param scale The scale parameter
-#' @param lower_bound The lower bound to be used (current time)
-#'
-#' @return Estimate(s) from the conditional loglogistic distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_llogis(rnd = 0.5,shape = 1,scale = 1,lower_bound = 1)
-
-qcond_llogis <- function(rnd = 0.5, shape, scale, lower_bound=0) {
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
-  }
-  
-  scale * ((1/(1-rnd))*(rnd + (lower_bound/scale)^shape))^(1/shape) - lower_bound
-}
-
-#' Conditional quantile function for lognormal distribution 
-#'
-#' @param rnd Vector of quantiles
-#' @param meanlog The meanlog parameter
-#' @param sdlog The sdlog parameter
-#' @param lower_bound The lower bound to be used (current time)
-#' @param s_obs is the survival observed up to lower_bound time,
-#'  normally defined from time 0 as 1 - plnorm(q = lower_bound, meanlog, sdlog) but may be different if parametrization has changed previously
-#'
-#' @importFrom stats qnorm
-#'
-#' @return Estimate(s) from the conditional lognormal distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_lnorm(rnd = 0.5, meanlog = 1,sdlog = 1,lower_bound = 1, s_obs=0.8)
-
-qcond_lnorm <- function(rnd = 0.5, meanlog, sdlog, lower_bound=0, s_obs) {
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
-  }
-  
-  exp(meanlog + sdlog * qnorm(1 - s_obs * (1-rnd))) - lower_bound
-}
-
-
-#' Conditional quantile function for normal distribution 
-#'
-#' @param rnd Vector of quantiles
-#' @param mean The mean parameter
-#' @param sd The sd parameter
-#' @param lower_bound The lower bound to be used (current time)
-#' @param s_obs is the survival observed up to lower_bound time,
-#'  normally defined from time 0 as 1 - pnorm(q = lower_bound, mean, sd) but may be different if parametrization has changed previously
-#'
-#' @importFrom stats qnorm
-#'
-#' @return Estimate(s) from the conditional normal distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_norm(rnd = 0.5, mean = 1,sd = 1,lower_bound = 1, s_obs=0.8)
-
-qcond_norm <- function(rnd = 0.5, mean,sd, lower_bound=0, s_obs) {
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
-  }
-
-  qnorm(1 - s_obs*(1-rnd),mean,sd) - lower_bound
-}
-
-
-#' Conditional quantile function for gamma distribution 
-#'
-#' @param rnd Vector of quantiles
-#' @param rate The rate parameter
-#' @param shape The shape parameter
-#' @param lower_bound The lower bound to be used (current time)
-#' @param s_obs is the survival observed up to lower_bound time,
-#'  normally defined from time 0 as 1 - pgamma(q = lower_bound, rate, shape) but may be different if parametrization has changed previously
-#'
-#' @importFrom stats qgamma
-#'
-#' @return Estimate(s) from the conditional gamma distribution based on given parameters
-#'
-#' @export
-#'
-#' @examples
-#' qcond_gamma(rnd = 0.5, shape = 1.06178, rate = 0.01108,lower_bound = 1, s_obs=0.8)
-
-qcond_gamma <- function(rnd = 0.5, shape,rate, lower_bound=0, s_obs) {
-  if(rnd <0 | rnd > 1){
-    stop("rnd is <0 or >1")
-  }
-  
-  qgamma(1 - s_obs*(1-rnd),rate = rate, shape = shape) - lower_bound
-}
-
-
-
-#' Draw Time-to-Event with Time-Dependent Covariates and Luck Adjustment
-#'
-#' Simulate a time-to-event (TTE) from a parametric distribution with parameters varying over time.
-#' User provides parameter functions and distribution name. The function uses internal survival and
-#' conditional quantile functions, plus luck adjustment to simulate the event time. See
-#' the vignette on avoiding cycles for an example in a model.
-#' 
-#'
-#' @param luck Numeric between 0 and 1. Initial random quantile (luck).
-#' @param a_fun Function of time .time returning the first distribution parameter (e.g., rate, shape, meanlog).
-#' @param b_fun Function of time .time returning the second distribution parameter (e.g., scale, sdlog). Defaults to a function returning NA.
-#' @param dist Character string specifying the distribution. Supported: "exp", "gamma", "lnorm", "norm", "weibull", "llogis", "gompertz".
-#' @param dt Numeric. Time step increment to update parameters and survival. Default 0.1.
-#' @param max_time Numeric. Max allowed event time to prevent infinite loops. Default 100.
-#' @param return_luck Boolean. If TRUE, returns a list with tte and luck (useful if max_time caps TTE)
-#' @param start_time Numeric. Time to use as a starting point of reference (e.g., curtime).
-#'
-#' @return Numeric. Simulated time-to-event.
-#' 
-#' 
-#' @importFrom flexsurv pllogis pgompertz pweibullPH
-#' @importFrom stats pexp pgamma plnorm pnorm pweibull
-#' 
-#' @export
-#' 
-#' @details The objective of this function is to avoid the user to have cycle events
-#' with the only scope of updating some variables that depend on time and re-evaluate
-#' a TTE. The idea is that this function should only be called at start and when 
-#' an event impacts a variable (e.g., stroke event impacting death TTE), in which case
-#' it would need to be called again at that point. In that case, the user would need to 
-#' call e.g., `a <- qtimecov` with `max_time = curtime, return_luck = TRUE` arguments,
-#' and then call it again with no max_time, return_luck = FALSE, and
-#' `luck = a$luck, start_time=a$tte` (so there is no need to add curtime to the resulting time).
-#' 
-#' It's recommended to play with `dt` argument to balance running time and precision of the estimates.
-#' For example, if we know we only update the equation annually (not continuously),
-#' then we could just set `dt = 1`, which would make computations faster.
-#'
-#' @examples
-#'
-#' param_fun_factory <- function(p0, p1, p2, p3) {
-#'   function(.time) p0 + p1*.time + p2*.time^2 + p3*(floor(.time) + 1)
-#' }
-#' 
-#' set.seed(42)
-#' 
-#' # 1. Exponential Example
-#' rate_exp <- param_fun_factory(0.1, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = rate_exp,
-#'   dist = "exp"
-#' )
-#' 
-#' 
-#' # 2. Gamma Example
-#' shape_gamma <- param_fun_factory(2, 0, 0, 0)
-#' rate_gamma <- param_fun_factory(0.2, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = shape_gamma,
-#'   b_fun = rate_gamma,
-#'   dist = "gamma"
-#' )
-#' 
-#' 
-#' # 3. Lognormal Example
-#' meanlog_lnorm <- param_fun_factory(log(10) - 0.5*0.5^2, 0, 0, 0)
-#' sdlog_lnorm <- param_fun_factory(0.5, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = meanlog_lnorm,
-#'   b_fun = sdlog_lnorm,
-#'   dist = "lnorm"
-#' )
-#' 
-#' 
-#' # 4. Normal Example
-#' mean_norm <- param_fun_factory(10, 0, 0, 0)
-#' sd_norm <- param_fun_factory(2, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = mean_norm,
-#'   b_fun = sd_norm,
-#'   dist = "norm"
-#' )
-#' 
-#' 
-#' # 5. Weibull Example
-#' shape_weibull <- param_fun_factory(2, 0, 0, 0)
-#' scale_weibull <- param_fun_factory(10, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = shape_weibull,
-#'   b_fun = scale_weibull,
-#'   dist = "weibull"
-#' )
-#' 
-#' 
-#' # 6. Loglogistic Example
-#' shape_llogis <- param_fun_factory(2.5, 0, 0, 0)
-#' scale_llogis <- param_fun_factory(7.6, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = shape_llogis,
-#'   b_fun = scale_llogis,
-#'   dist = "llogis"
-#' )
-#' 
-#' 
-#' # 7. Gompertz Example
-#' shape_gomp <- param_fun_factory(0.01, 0, 0, 0)
-#' rate_gomp <- param_fun_factory(0.091, 0, 0, 0)
-#' qtimecov(
-#'   luck = runif(1),
-#'   a_fun = shape_gomp,
-#'   b_fun = rate_gomp,
-#'   dist = "gompertz"
-#' )
-#'
-#' #Time varying example, with change at time 8
-#' rate_exp <- function(.time) 0.1 + 0.01*.time * 0.00001*.time^2
-#' rate_exp2 <- function(.time) 0.2 + 0.02*.time
-#' time_change <- 8
-#' init_luck <- 0.95
-#'
-#' a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005,
-#'                       max_time = time_change, return_luck = TRUE)
-#' qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)
-#' 
-#' 
-#' #An example of how it would work in the model, this would also work with time varying covariates!
-#' rate_exp <- function(.time) 0.1
-#' rate_exp2 <- function(.time) 0.2
-#' rate_exp3 <- function(.time) 0.3
-#' time_change <- 10 #evt 1
-#' time_change2 <- 15 #evt2
-#' init_luck <- 0.95
-#' #at start, we would just draw TTE
-#' qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005)
-#' 
-#' #at event in which rate changes (at time 10) we need to do this:
-#' a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005,
-#'                       max_time = time_change, return_luck = TRUE)
-#' new_luck <- a$luck
-#' qtimecov(luck = new_luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)
-#' 
-#' #at second  event in which rate changes again (at time 15) we need to do this:
-#' a <- qtimecov(luck = new_luck,a_fun = rate_exp2,dist = "exp", dt = 0.005,
-#'                       max_time = time_change2, return_luck = TRUE, start_time=a$tte)
-#' new_luck <- a$luck
-#' #final TTE is
-#' qtimecov(luck = new_luck,a_fun = rate_exp3,dist = "exp", dt = 0.005, start_time=a$tte)
-#'   
-
-qtimecov <- function(
-    luck,
-    a_fun,
-    b_fun = function(.time) NA,
-    dist,
-    dt = 0.1,
-    max_time = 100,
-    return_luck = FALSE,
-    start_time = 0
-) {
-  # Internal survival and conditional quantile functions (require flexsurv for some)
-  sf_fun <- switch(dist,
-                   exp       = function(.time, rate, ...) 1 - pexp(.time, rate),
-                   gamma     = function(.time, shape, rate) 1 - pgamma(.time, shape = shape, rate = rate),
-                   lnorm     = function(.time, meanlog, sdlog) 1 - plnorm(.time, meanlog, sdlog),
-                   norm      = function(.time, mean, sd) 1 - pnorm(.time, mean, sd),
-                   weibull   = function(.time, shape, scale) 1 - pweibull(.time, shape, scale),
-                   weibullPH = function(.time, shape, scale) 1 - flexsurv::pweibullPH(.time, shape, scale),
-                   llogis    = function(.time, shape, scale) flexsurv::pllogis(q = .time, shape = shape, scale = scale, lower.tail = FALSE),
-                   gompertz  = function(.time, shape, rate) flexsurv::pgompertz(q = .time, shape = shape, rate = rate, lower.tail = FALSE),
-                   stop("Unsupported distribution")
-  )
-  
-  qcond_fun <- switch(dist,
-                      exp       = qcond_exp,
-                      gamma     = qcond_gamma,
-                      lnorm     = qcond_lnorm,
-                      norm      = qcond_norm,
-                      weibull   = qcond_weibull,
-                      weibullPH = qcond_weibullPH,
-                      llogis    = qcond_llogis,
-                      gompertz  = qcond_gompertz,
-                      stop("Unsupported distribution")
-  )
-  
-  .time <- start_time
-  a_curr <- a_fun(.time)
-  b_curr <- b_fun(.time)
-  surv_prev <- sf_fun(.time, a_curr, b_curr)
-  
-  
-  # Early exit: check whether event occurs in [0, dt]
-  residual_tte <- switch(dist,
-                         exp       = qcond_exp(luck, rate = a_curr),
-                         gamma     = qcond_gamma(luck, rate = b_curr, shape = a_curr, lower_bound = start_time, s_obs = surv_prev),
-                         lnorm     = qcond_lnorm(luck, meanlog = a_curr, sdlog = b_curr, lower_bound = start_time, s_obs = surv_prev),
-                         norm      = qcond_norm(luck, mean = a_curr, sd = b_curr, lower_bound = start_time, s_obs = surv_prev),
-                         weibull   = qcond_weibull(luck, shape = a_curr, scale = b_curr, lower_bound = start_time),
-                         weibullPH = qcond_weibullPH(luck, shape = a_curr, scale = b_curr, lower_bound = start_time),
-                         llogis    = qcond_llogis(luck, shape = a_curr, scale = b_curr, lower_bound = start_time),
-                         gompertz  = qcond_gompertz(luck, shape = a_curr, rate = b_curr, lower_bound = start_time),
-                         stop("Unsupported distribution")
-  )
-  
-  if (residual_tte <= dt) {
-    if (return_luck == TRUE) {
-      return(list(tte = residual_tte + .time, luck = luck))
-    } else {
-      return(residual_tte)
+  env$attempt_block <- function(patient_id = NULL, priority = 1L, start_time = NULL) {
+    if (is.null(patient_id)) {
+      patient_id <- tryCatch(get("i", envir = parent.frame(), inherits = TRUE),
+                             error = function(e) stop("patient_id not provided and 'i' not found in parent frame"))
     }
+    if (is.null(start_time)) {
+      start_time <- tryCatch(get("curtime", envir = parent.frame(), inherits = TRUE),
+                             error = function(e) stop("start_time not provided and 'curtime' not found in parent frame"))
+    }
+    if (!is.numeric(patient_id) || length(patient_id) != 1) stop("patient_id must be a single number")
+    if (!is.numeric(priority)   || length(priority)   != 1) stop("priority must be a single number")
+    if (!is.numeric(start_time) || length(start_time) != 1) stop("start_time must be a single number")
+    discrete_resource_attempt_block_cpp(env$.ptr, as.integer(patient_id), as.integer(priority), as.numeric(start_time))
   }
   
-  repeat {
-    .time <- .time + dt
-
-    # Correctly compute survival at .time - dt for luck adjustment
-    surv_prev <- sf_fun(.time - dt, a_curr, b_curr)
-    surv_curr <- sf_fun(.time, a_curr, b_curr)
-    a_curr <- a_fun(.time)
-    b_curr <- b_fun(.time)
-    
-    # Use condq = TRUE for conditional quantiles
-    luck <- luck_adj(prevsurv = surv_prev, cursurv = surv_curr, luck = luck, condq = TRUE)
-    
-    residual_tte <- switch(dist,
-                           exp       = qcond_exp(luck, rate = a_curr),
-                           gamma     = qcond_gamma(luck, rate = b_curr, shape = a_curr, lower_bound = .time - dt, s_obs = surv_prev),
-                           lnorm     = qcond_lnorm(luck, meanlog = a_curr, sdlog = b_curr, lower_bound = .time - dt, s_obs = surv_prev),
-                           norm      = qcond_norm(luck, mean = a_curr, sd = b_curr, lower_bound = .time - dt, s_obs = surv_prev),
-                           weibull   = qcond_weibull(luck, shape = a_curr, scale = b_curr, lower_bound = .time - dt),
-                           weibullPH = qcond_weibullPH(luck, shape = a_curr, scale = b_curr, lower_bound = .time - dt),
-                           llogis    = qcond_llogis(luck, shape = a_curr, scale = b_curr, lower_bound = .time - dt),
-                           gompertz  = qcond_gompertz(luck, shape = a_curr, rate = b_curr, lower_bound = .time - dt),
-                           stop("Unsupported distribution")
-    )
-    
-    total_tte <- .time + residual_tte
-    
-    if (residual_tte <= dt || total_tte <= .time || .time >= max_time) {
-      if (return_luck == TRUE) {
-        return(list(tte = min(total_tte, max_time), luck = luck))
-      } else {
-        return(min(total_tte, max_time))
-      }
+  env$attempt_free <- function(patient_id = NULL, remove_all = FALSE) {
+    if (is.null(patient_id)) {
+      patient_id <- tryCatch(get("i", envir = parent.frame(), inherits = TRUE),
+                             error = function(e) stop("patient_id not provided and 'i' not found in parent frame"))
     }
+    if (!is.numeric(patient_id) || length(patient_id) != 1) stop("patient_id must be a single number")
+    if (!is.logical(remove_all) || length(remove_all) != 1) stop("remove_all must be a single logical value")
+    discrete_resource_attempt_free_cpp(env$.ptr, as.integer(patient_id), remove_all)
+    invisible(NULL)
   }
+  
+  env$attempt_free_if_using <- function(patient_id = NULL, remove_all = FALSE) {
+    if (is.null(patient_id)) {
+      patient_id <- tryCatch(get("i", envir = parent.frame(), inherits = TRUE),
+                             error = function(e) stop("patient_id not provided and 'i' not found in parent frame"))
+    }
+    if (!is.numeric(patient_id) || length(patient_id) != 1) stop("patient_id must be a single number")
+    if (!is.logical(remove_all) || length(remove_all) != 1) stop("remove_all must be a single logical value")
+    discrete_resource_attempt_free_if_using_cpp(env$.ptr, as.integer(patient_id), remove_all)
+    invisible(NULL)
+  }
+  
+  env$next_patient_in_line <- function(n = 1L) {
+    if (!is.numeric(n) || length(n) != 1 || n < 1) stop("n must be a single positive integer")
+    discrete_resource_next_patient_in_line_cpp(env$.ptr, as.integer(n))
+  }
+  
+  env$modify_priority <- function(patient_id, new_priority) {
+    if (!is.numeric(patient_id) || length(patient_id) != 1) stop("patient_id must be a single number")
+    if (!is.numeric(new_priority) || length(new_priority) != 1) stop("new_priority must be a single number")
+    discrete_resource_modify_priority_cpp(env$.ptr, as.integer(patient_id), as.integer(new_priority))
+    invisible(NULL)
+  }
+  
+  env$add_resource <- function(n_to_add) {
+    if (!is.numeric(n_to_add) || length(n_to_add) != 1 || n_to_add < 1) stop("n_to_add must be a single positive integer")
+    discrete_resource_add_resource_cpp(env$.ptr, as.integer(n_to_add))
+    invisible(NULL)
+  }
+  
+  env$remove_resource <- function(n_to_remove, current_time = NULL) {
+    if (is.null(current_time)) {
+      current_time <- tryCatch(get("curtime", envir = parent.frame(), inherits = TRUE),
+                               error = function(e) stop("current_time not provided and 'curtime' not found in parent frame"))
+    }
+    if (!is.numeric(n_to_remove) || length(n_to_remove) != 1 || n_to_remove < 1) stop("n_to_remove must be a single positive integer")
+    if (!is.numeric(current_time) || length(current_time) != 1) stop("current_time must be a single number")
+    discrete_resource_remove_resource_cpp(env$.ptr, as.integer(n_to_remove), as.numeric(current_time))
+    invisible(NULL)
+  }
+  
+  class(env) <- "resource_discrete"
+  env
 }
 
-
+#' Clone independent discrete resources
+#'
+#' @param x discrete resource created with resource_discrete()
+#' @param n Number of independent clones to be generated
+#' 
+#' @return List of independent clones of discrete resource envs (even for n = 1)
+#'
+#' @export
+#'
+#' 
+discrete_resource_clone <- function(x, n = 1) {
+  if (!is.environment(x) || is.null(x$.ptr)) {
+    stop("Expected a resource wrapper environment with field `.ptr`.", call. = FALSE)
+  }
+  if (!is.numeric(n) || length(n) != 1 || n < 1) {
+    stop("n must be a single integer >= 1", call. = FALSE)
+  }
+  xptrs <- discrete_resource_clone_xptrs_cpp(x, as.integer(n))
+  lapply(xptrs, .resource_discrete_from_xptr)
+}
\ No newline at end of file
