@@ -1,11 +1,11 @@
 test_that("Vector discounting equal to non-vector for single-length elements", {
   #Ongoing
   expect_equal(disc_ongoing_v(lcldr=0.035,lclprvtime=0.5, lclcurtime=3, lclval=2500),
-               disc_ongoing(lcldr=0.035,lclprvtime=0.5, lclcurtime=3, lclval=2500))
+               suppressWarnings(disc_ongoing(lcldr=0.035,lclprvtime=0.5, lclcurtime=3, lclval=2500)))
   
   #Instant
   expect_equal(disc_instant_v(lcldr=0.035, lclcurtime=3, lclval=2500),
-               disc_instant(lcldr=0.035, lclcurtime=3, lclval=2500))
+               suppressWarnings(disc_instant(lcldr=0.035, lclcurtime=3, lclval=2500)))
 
  })
 
@@ -14,19 +14,19 @@ test_that("Discounting works with no discounting", {
   #Ongoing
   expect_equal(disc_ongoing_v(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500),
                2500)
-  expect_equal(disc_ongoing(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500),
+  expect_equal(suppressWarnings(disc_ongoing(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500)),
                2500)
   
   #Instant
   expect_equal(disc_instant_v(lcldr=0, lclcurtime=2, lclval=2500),
                2500)
-  expect_equal(disc_instant(lcldr=0, lclcurtime=2, lclval=2500),
+  expect_equal(suppressWarnings(disc_instant(lcldr=0, lclcurtime=2, lclval=2500)),
                2500)
   
   #Cycle
   expect_equal(disc_cycle_v(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500, cyclelength=1/12, starttime=0),
                12*2500)
-  expect_equal(disc_cycle(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500, cyclelength=1/12, starttime=0),
+  expect_equal(suppressWarnings(disc_cycle(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500, cyclelength=1/12, starttime=0)),
                12*2500)
 })
 
@@ -36,44 +36,44 @@ test_that("Discounting works with odd numbers", {
   #Ongoing
   expect_equal(disc_ongoing_v(lcldr=0,lclprvtime=0, lclcurtime=Inf, lclval=2500),
                Inf)
-  expect_equal(disc_ongoing(lcldr=0,lclprvtime=0, lclcurtime=Inf, lclval=2500),
+  expect_equal(suppressWarnings(disc_ongoing(lcldr=0,lclprvtime=0, lclcurtime=Inf, lclval=2500)),
                Inf)
   expect_equal(disc_ongoing_v(lcldr=0.035,lclprvtime=0, lclcurtime=0, lclval=2500),
                0)
-  expect_equal(disc_ongoing(lcldr=0.035,lclprvtime=0, lclcurtime=0, lclval=2500),
+  expect_equal(suppressWarnings(disc_ongoing(lcldr=0.035,lclprvtime=0, lclcurtime=0, lclval=2500)),
                0)
   expect_equal(disc_ongoing_v(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=2500),
                0)
-  expect_equal(disc_ongoing(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=2500),
+  expect_equal(suppressWarnings(disc_ongoing(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=2500)),
                0)
   #Inf*0 gives NaN, while the element-wise function just check wehterh prevtime and curtime are equal
   expect_equal(disc_ongoing_v(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=Inf),
                NaN)
-  expect_equal(disc_ongoing(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=Inf),
+  expect_equal(suppressWarnings(disc_ongoing(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=Inf)),
                NaN) 
   
   #Instant
   expect_equal(disc_instant_v(lcldr=0, lclcurtime=Inf, lclval=2500),
                2500)
-  expect_equal(disc_instant(lcldr=0, lclcurtime=Inf, lclval=2500),
+  expect_equal(suppressWarnings(disc_instant(lcldr=0, lclcurtime=Inf, lclval=2500)),
                2500)
   expect_equal(disc_instant_v(lcldr=0.035, lclcurtime=0, lclval=2500),
                2500)
-  expect_equal(disc_instant(lcldr=0.035, lclcurtime=0, lclval=2500),
+  expect_equal(suppressWarnings(disc_instant(lcldr=0.035, lclcurtime=0, lclval=2500)),
                2500)
   expect_equal(disc_instant_v(lcldr=0.035, lclcurtime=5, lclval=2500),
-               2104.93292,tolerance  = 0.01)
-  expect_equal(disc_instant(lcldr=0.035, lclcurtime=5, lclval=2500),
-               2104.93292,tolerance  = 0.01)
+               2104.93292)
+  expect_equal(suppressWarnings(disc_instant(lcldr=0.035, lclcurtime=5, lclval=2500)),
+               2104.93292)
   #Inf*0 gives NaN
   expect_equal(disc_instant_v(lcldr=0.035, lclcurtime=5, lclval=Inf),
                Inf)
-  expect_equal(disc_instant(lcldr=0.035, lclcurtime=5, lclval=Inf),
+  expect_equal(suppressWarnings(disc_instant(lcldr=0.035, lclcurtime=5, lclval=Inf)),
                Inf) 
   expect_equal(disc_instant_v(lcldr=5, lclcurtime=5, lclval=2500),
-               0.32150206,tolerance  = 0.01)
-  expect_equal(disc_instant(lcldr=5, lclcurtime=5, lclval=2500),
-               0.32150206,tolerance  = 0.01) 
+               0.32150206)
+  expect_equal(suppressWarnings(disc_instant(lcldr=5, lclcurtime=5, lclval=2500)),
+               0.32150206) 
   
   #Cycle
   expect_equal(disc_cycle_v(lcldr=0,lclprvtime=1, lclcurtime=2, lclval=2500, cyclelength=1/12, starttime=1),
@@ -82,8 +82,7 @@ test_that("Discounting works with odd numbers", {
                2500)
   expect_equal(disc_cycle_v(lcldr=0.035,lclprvtime=0, lclcurtime=0, lclval=2500, cyclelength=2, starttime=0),
                2500)
-  # #Inf*0 gives 
-  # Test removed as it's returning Inf in systems iwthout long-double instead of NaN
+  # #Inf*0 gives NaN
   # expect_equal(disc_cycle_v(lcldr=0.035,lclprvtime=5, lclcurtime=5, lclval=Inf, cyclelength=1/12, starttime=0),
   #              NaN)
 })
@@ -93,20 +92,20 @@ test_that("Discounting works with odd numbers", {
 test_that("Vectorial discounting working as expected with vectors", {
   #Ongoing
   expect_equal(disc_ongoing_v(lcldr=0.035,lclprvtime=c(0.5,0.5,0.5), lclcurtime=c(3,3,3), lclval=c(0,1000,Inf)),
-               c(0,2354.66015,Inf),tolerance  = 0.01)
+               c(0,2354.66015,Inf))
   
   #Instant
   expect_equal(disc_instant_v(lcldr=0.035, lclcurtime=c(3,3,3), lclval=c(0,1000,Inf)),
-               c(0,901.9427,Inf),tolerance  = 0.01)
+               c(0,901.9427,Inf))
   
   #Cycle
   expect_equal(disc_cycle_v(lcldr=0.035, lclcurtime=c(3,3,3), lclval=c(0,1000,Inf),lclprvtime=c(0.5,0.5,0.5), cyclelength=c(1/12,1/12,1/12),starttime=c(0,0,0)),
-               c(0,28296.443022132232727,Inf),tolerance  = 0.01)  
+               c(0,28296.443022132232727,Inf))  
   
   expect_equal(disc_cycle_v(0.035,0,1,1,1000,0,5),1000)
-  expect_equal(disc_cycle_v(0.035,1,1,1,1000,0,5),966.1835748792273,tolerance  = 0.01)
+  expect_equal(disc_cycle_v(0.035,1,1,1,1000,0,5),966.1835748792273)
   expect_equal(disc_cycle_v(0.035,0,1,0,1000,0,5),1000)
-  expect_equal(disc_cycle_v(0.035,0,1,2,1000,0,5),1966.1835748792273,tolerance  = 0.01)
+  expect_equal(disc_cycle_v(0.035,0,1,2,1000,0,5),1966.1835748792273)
 })
 
 test_that("Create indicators works correctly",{
@@ -199,7 +198,7 @@ test_that("Pick values vectorized work correctly",{
                indicator_sens_binary = FALSE,
                sens_iterator = 5,
                distributions = list("rnorm","rnorm","rdirichlet"),
-               covariances = list(0.1,0.1,NULL) ),
+               covariances = list(0.1,0.1,NULL), deploy = FALSE ),
     list(util = 2, util2 = 3, dirichlet_vector = c(0.36, 0.54, 0.1))
     
   )
@@ -211,7 +210,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = FALSE,
       sens_ind = FALSE,
-      indicator=c(1,0)
+      indicator=c(1,0), deploy = FALSE
     ),
     list(0,0)
   )
@@ -223,7 +222,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = FALSE,
       sens_ind = TRUE,
-      indicator=c(1,0)
+      indicator=c(1,0), deploy = FALSE
     ),
     list(2,0)
   )
@@ -235,7 +234,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = FALSE,
       sens_ind = TRUE,
-      indicator=c(0,1)
+      indicator=c(0,1), deploy = FALSE
     ),
     list(0,3)
   )
@@ -247,7 +246,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = FALSE,
       sens_ind = TRUE,
-      indicator=c(1,1)
+      indicator=c(1,1), deploy = FALSE
     ),
     list(2,3)
   )
@@ -259,7 +258,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = TRUE,
       sens_ind = TRUE,
-      indicator=c(1,1)
+      indicator=c(1,1), deploy = FALSE
     ),
     list(2,3)
   )
@@ -271,7 +270,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = TRUE,
       sens_ind = TRUE,
-      indicator=c(1,5)
+      indicator=c(1,5), deploy = FALSE
     )
   )
   
@@ -282,7 +281,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = 5,
       sens_ind = TRUE,
-      indicator=c(1,1)
+      indicator=c(1,1), deploy = FALSE
     )
   )
   
@@ -293,7 +292,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = TRUE,
       sens_ind = 3,
-      indicator=c(1,1)
+      indicator=c(1,1), deploy = FALSE
     )
   )
   
@@ -304,7 +303,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = TRUE,
       sens_ind = TRUE,
-      indicator=c(0,0)
+      indicator=c(0,0), deploy = FALSE
     ),
     {list(draw_tte(1,'norm',0,0.1,seed=1),draw_tte(1,'norm',0,0.1,seed=2))}
   )
@@ -316,7 +315,7 @@ test_that("Pick values vectorized work correctly",{
       sens = c(2,3),
       psa_ind = TRUE,
       sens_ind = TRUE,
-      indicator=c(1,0)
+      indicator=c(1,0), deploy = FALSE
     ),
     {list(2,draw_tte(1,'norm',0,0.1,seed=2))}
   )
@@ -395,82 +394,80 @@ test_that("Model Reactions Interactivity summary can be created",{
     
     l <- 9
     
-    modify_item(list(afsa=ifelse(TRUE,"asda",NULL)))
+    afsa=ifelse(TRUE,"asda",NULL)
     
-    modify_item_seq(list(
+
+      o_exn = o_exn + 1
       
-      o_exn = o_exn + 1,
+      a = NULL
       
-      a = NULL,
+      b = if(a){"CZ"}else{"AW"}
       
-      b = if(a){"CZ"}else{"AW"},
+      rnd_prob_exn_sev = runif(1)
       
-      rnd_prob_exn_sev = runif(1),
+      exn_sev = rnd_prob_exn_sev <= p_sev
       
-      exn_sev = rnd_prob_exn_sev <= p_sev,
+      o_exn_mod = o_exn_mod + if(exn_sev) { 0 } else { 1 }
       
-      o_exn_mod = o_exn_mod + if(exn_sev) { 0 } else { 1 },
+      o_exn_sev = o_exn_sev + if(exn_sev) { 1 } else { 0 }
       
-      o_exn_sev = o_exn_sev + if(exn_sev) { 1 } else { 0 },
+      o_rec_time_without_exn = (o_exn == 0) * 1
       
-      o_rec_time_without_exn = (o_exn == 0) * 1,
+      o_rec_time_without_exn_sev = (o_exn_sev == 0) * 1
       
-      o_rec_time_without_exn_sev = (o_exn_sev == 0) * 1,
+      o_c_exn = if(exn_sev) { c_sev } else { c_mod }
+  
+      o_other_c_exn_mod = if(exn_sev) { 0 } else { c_mod }
       
-      o_c_exn = if(exn_sev) { c_sev } else { c_mod },
+      o_other_c_exn_sev = if(exn_sev) { c_sev } else { 0 }
       
-      o_other_c_exn_mod = if(exn_sev) { 0 } else { c_mod },
+      o_qloss_exn = -if(exn_sev) { q_sev } else { q_mod }
       
-      o_other_c_exn_sev = if(exn_sev) { c_sev } else { 0 },
+      o_other_qloss_exn_mod = -if(exn_sev) { 0 } else { q_mod }
+    
+      o_other_qloss_exn_sev = -if(exn_sev) { q_sev } else { 0 }
       
-      o_qloss_exn = -if(exn_sev) { q_sev } else { q_mod },
+      o_qloss_cg_exn = -if(exn_sev) { q_cg_sev } else { q_cg_mod }
+    
+      o_other_qloss_cg_exn_mod = -if(exn_sev) { 0 } else { q_cg_mod }
       
-      o_other_qloss_exn_mod = -if(exn_sev) { 0 } else { q_mod },
+      o_other_qloss_cg_exn_sev = -if(exn_sev) { q_cg_sev } else { 0 }
       
-      o_other_qloss_exn_sev = -if(exn_sev) { q_sev } else { 0 },
+      o_q = utility
       
-      o_qloss_cg_exn = -if(exn_sev) { q_cg_sev } else { q_cg_mod },
+      o_other_q_gold1 = if(gold == 1) { utility } else { 0 }
       
-      o_other_qloss_cg_exn_mod = -if(exn_sev) { 0 } else { q_cg_mod },
+      o_other_q_gold2 = if(gold == 2) { utility } else { 0 }
       
-      o_other_qloss_cg_exn_sev = -if(exn_sev) { q_cg_sev } else { 0 },
+      o_other_q_gold3 = if(gold == 3) { utility } else { 0 }
       
-      o_q = utility,
+      o_other_q_gold4 = if(gold == 4) { utility } else { 0 }
       
-      o_other_q_gold1 = if(gold == 1) { utility } else { 0 },
+      o_other_q_on_dup = if(on_dup) { utility } else { 0 }
+    
+      n_exn = n_exn + 1
       
-      o_other_q_gold2 = if(gold == 2) { utility } else { 0 },
+      n_exn_mod = n_exn_mod + (1 - exn_sev)
       
-      o_other_q_gold3 = if(gold == 3) { utility } else { 0 },
+      n_exn_sev = n_exn_sev + exn_sev
       
-      o_other_q_gold4 = if(gold == 4) { utility } else { 0 },
+      u_adj_exn_lt = u_adj_exn_lt + if(exn_sev) { u_adj_sev_lt } else { u_adj_mod_lt }
       
-      o_other_q_on_dup = if(on_dup) { utility } else { 0 },
+      utility = u_gold - u_adj_exn_lt - u_mace_lt
       
-      n_exn = n_exn + 1,
-      
-      n_exn_mod = n_exn_mod + (1 - exn_sev),
-      
-      n_exn_sev = n_exn_sev + exn_sev,
-      
-      u_adj_exn_lt = u_adj_exn_lt + if(exn_sev) { u_adj_sev_lt } else { u_adj_mod_lt },
-      
-      utility = u_gold - u_adj_exn_lt - u_mace_lt,
-      
-      o_rec_utility = utility,
+      o_rec_utility = utility
       
       rnd_exn = runif(1)
-      
-    ))
+    
     
     if(a==1){
-      modify_item(list(a=list(6+b)))
+      a=list(6+b)
       
       modify_event(list(e_exn = curtime + 14 / days_in_year + qexp(rnd_exn, r_exn)))
     } else{
       modify_event(list(e_exn = curtime + 14 / days_in_year + qexp(rnd_exn, r_exn)))
       if(a>6){
-        modify_item(list(a=8))
+        a=8
       }
       
     }
@@ -484,7 +481,7 @@ test_that("Model Reactions Interactivity summary can be created",{
     
   })
   
-  expect_length(ast_as_list(expr),13) 
+  expect_length(ast_as_list(expr),43) 
   
   expect_type(ast_as_list(expr),"list")
   
@@ -497,7 +494,7 @@ test_that("Model Reactions Interactivity summary can be created",{
   a <- add_reactevt(name_evt="example",
                     input={
                       a <- 5
-                      modify_item(list(w=5))
+                      w=5
                     })
   
   
@@ -530,39 +527,26 @@ test_that("add_tte works as expected", {
   expect_equal(result$control$evts, evts)
 })
 
-test_that("modify_item modifies input items correctly", {
-  input_list_arm <- environment()
-  input_list_arm$qaly_default_instant = 100
-  input_list_arm$debug = FALSE
-  input_list_arm$accum_backwards = FALSE
-
-  modify_item(list("qaly_default_instant" = 200))
-  expect_equal(input_list_arm$qaly_default_instant, 200)
-  
-  modify_item(list(new_cost = 300))
-  expect_equal(input_list_arm$new_cost, 300)
-})
-
 test_that("modify_event modifies events correctly", {
   input_list_arm <- environment()
   input_list_arm$qaly_default_instant = 100
   input_list_arm$debug = FALSE
   input_list_arm$cur_evtlist = FALSE
   input_list_arm$accum_backwards = FALSE
-  input_list_arm$cur_evtlist = c(ae = 5, nat.death = 100)
-
+  event_queue <- queue_create(c("ae","new_event","nat.death","nonexistent"))
+  new_event(c(ae=5,nat.death=10),event_queue,1)
 
   # Modify an existing event
-  modify_event(list(ae = 10))
-  expect_equal(input_list_arm$cur_evtlist[["ae"]], 10)
+  modify_event(list(ae = 10), create_if_missing = TRUE, event_queue,1)
+  expect_equal(get_event("ae",event_queue,1),10)
   
   # Create new event if not exists
-  modify_event(list(new_event = 50), create_if_null = TRUE)
-  expect_equal(input_list_arm$cur_evtlist[["new_event"]], 50)
+  modify_event(list(new_event = 50), create_if_missing = TRUE, event_queue,1)
+  expect_equal(get_event("new_event",event_queue,1), 50)
   
   # Ignore non-existent event
-  expect_warning(modify_event(list(nonexistent = 20), create_if_null = FALSE))
-  expect_error(input_list_arm$cur_evtlist[["nonexistent"]])
+  expect_no_condition(modify_event(list(nonexistent = 20), create_if_missing = FALSE, event_queue,1))
+  expect_error(get_event("nonexistent",event_queue,1))
 })
 
 test_that("new_event adds new events correctly", {
@@ -572,12 +556,12 @@ test_that("new_event adds new events correctly", {
   input_list_arm$cur_evtlist = FALSE
   input_list_arm$accum_backwards = FALSE
   input_list_arm$cur_evtlist = c()
+  event_queue <- queue_create(c("ae","not_numeric"))
+  new_event(c(ae=5),event_queue,1)
   
-  new_event(list("ae" = 5))
-  expect_equal(input_list_arm$cur_evtlist[["ae"]], 5)
+  expect_equal(get_event("ae",event_queue,1), 5)
   
-  expect_error(new_event(list("not_numeric" = "five")), 
-               "New event times are not all numeric, please review")
+  expect_error(new_event(c("not_numeric" = "five")))
 })
 
 
@@ -599,30 +583,6 @@ test_that("replicate_profiles works correctly", {
 })
 
 
-test_that("modify_item_seq works sequentially", {
-  
-  input_list_arm <- environment()
-  input_list_arm$a <- 1
-  input_list_arm$b <- 2
-  input_list_arm$curtime <- 1
-  input_list_arm$accum_backwards <- FALSE
-  input_list_arm$debug <- FALSE
-  
-  # Test sequential modification
-  modify_item_seq(list(a = 3, b = a + 2))
-  expect_equal(input_list_arm$a, 3)
-  expect_equal(input_list_arm$b, 5)
-  
-  # Test debug mode
-  input_list_arm$debug <- TRUE
-  input_list_arm$log_list <- list()
-  modify_item_seq(list(a = 4, c = b * 2))
-  expect_equal(input_list_arm$a, 4)
-  expect_equal(input_list_arm$c, 10)
-  expect_true(length(input_list_arm$log_list) > 0)
-})
-
-
 test_that("add_reactevt adds reactions correctly", {
   # Create an empty data list
   data_list <- list()
@@ -706,10 +666,10 @@ test_that("qtimecov returns numeric scalar within bounds", {
   rate_exp <- param_fun_factory(0.1, 0, 0, 0)
   set.seed(1)
   tte <- qtimecov(runif(1), a_fun = rate_exp, dist = "exp")
-  expect_type(tte, "double")
-  expect_length(tte, 1)
-  expect_gt(tte, 0)
-  expect_lt(tte, 100)
+  expect_type(tte$tte, "double")
+  expect_length(tte, 2)
+  expect_gt(tte$tte, 0)
+  expect_lt(tte$tte, 100)
 })
 
 test_that("qtimecov works for all supported distributions", {
@@ -722,87 +682,87 @@ test_that("qtimecov works for all supported distributions", {
   rate_exp <- param_fun_factory(0.1, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = rate_exp, dist = "exp"))
   expect_equal(qtimecov(luck = 0.5,a_fun = rate_exp,dist = "exp", dt = 0.001
-  ),qexp(0.5,0.1), tolerance = 0.01)
+  )$tte,qexp(0.5,0.1), tolerance = 0.01)
   
   # 2. Gamma
   shape <- param_fun_factory(2, 0, 0, 0)
   rate <- param_fun_factory(0.2, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = rate, dist = "gamma"))
   expect_equal(qtimecov(luck = 0.5,a_fun = shape, b_fun = rate, dist = "gamma", dt = 0.001
-  ),qgamma(0.5,2,0.2), tolerance = 0.01)
+  )$tte,qgamma(0.5,2,0.2), tolerance = 0.01)
   
   # 3. Lognormal
   meanlog <- param_fun_factory(log(10) - 0.5^2 / 2, 0, 0, 0)
   sdlog <- param_fun_factory(0.5, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = meanlog, b_fun = sdlog, dist = "lnorm"))
-  expect_equal(qtimecov(0.5, a_fun = meanlog, b_fun = sdlog, dist = "lnorm",dt=0.01),
+  expect_equal(qtimecov(0.5, a_fun = meanlog, b_fun = sdlog, dist = "lnorm",dt=0.01)$tte,
                qlnorm(0.5,log(10) - 0.5^2 / 2,0.5), tolerance = 0.01)
   
   # 4. Normal
   mean <- param_fun_factory(10, 0, 0, 0)
   sd <- param_fun_factory(2, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = mean, b_fun = sd, dist = "norm"))
-  expect_equal(qtimecov(0.5, a_fun = mean, b_fun = sd, dist = "norm",dt=0.01),
+  expect_equal(qtimecov(0.5, a_fun = mean, b_fun = sd, dist = "norm",dt=0.01)$tte,
                qnorm(0.5,10,2), tolerance = 0.01)
   
   # 5. Weibull
   shape <- param_fun_factory(2, 0, 0, 0)
   scale <- param_fun_factory(10, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = scale, dist = "weibull"))
-  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "weibull",dt=0.01),
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "weibull",dt=0.01)$tte,
                qweibull(0.5,2,10), tolerance = 0.01)
   
   # 5.1 WeibullPH
   shape <- param_fun_factory(2, 0, 0, 0)
   scale <- param_fun_factory(0.01, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = scale, dist = "weibullPH"))
-  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "weibullPH",dt=0.01),
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "weibullPH",dt=0.01)$tte,
                flexsurv::qweibullPH(0.5,2,0.01), tolerance = 0.01)
   
   # 6. Loglogistic
   shape <- param_fun_factory(2.5, 0, 0, 0)
   scale <- param_fun_factory(7.6, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = scale, dist = "llogis"))
-  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "llogis",dt=0.01),
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "llogis",dt=0.01)$tte,
                flexsurv::qllogis(0.5,2.5,7.6), tolerance = 0.01)
   
   # 7. Gompertz
   shape <- param_fun_factory(0.01, 0, 0, 0)
   rate <- param_fun_factory(0.091, 0, 0, 0)
   expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = rate, dist = "gompertz"))
-  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = rate, dist = "gompertz",dt=0.01),
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = rate, dist = "gompertz",dt=0.01)$tte,
                qgompertz(0.5,0.01,0.091), tolerance = 0.01)
   
   rate_exp <- function(t) 0.1
   init_luck <- 0.95
-  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001),{
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001, return_luck = TRUE,max_time = 10)
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001)$tte,{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001,max_time = 10)
     
-    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)},
+    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)$tte},
     tolerance = 0.01)
   
   init_luck <- 0.99
-  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001),{
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001, return_luck = TRUE,max_time = 5)
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001)$tte,{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001,max_time = 5)
     
-    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)},
+    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)$tte},
     tolerance = 0.01)
   
   
   init_luck <- 0.3
-  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001),{
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001, return_luck = TRUE,max_time = 1)
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001)$tte,{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001,max_time = 1)
     
-    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)},
+    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)$tte},
     tolerance = 0.01)
   
   
   
   init_luck <- 0.3
-  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,b_fun = function(t) 90000 ,dist = "weibull", dt = 0.001),{
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,b_fun = function(t) 90000,dist = "weibull", dt = 0.001, return_luck = TRUE,max_time = 1)
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,b_fun = function(t) 90000 ,dist = "weibull", dt = 0.001)$tte,{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,b_fun = function(t) 90000,dist = "weibull", dt = 0.001,max_time = 1)
     
-    qtimecov(luck = a$luck,a_fun = rate_exp,b_fun = function(t) 90000,dist = "weibull", dt = 0.001, start_time=a$tte)},
+    qtimecov(luck = a$luck,a_fun = rate_exp,b_fun = function(t) 90000,dist = "weibull", dt = 0.001, start_time=a$tte)$tte},
     tolerance = 0.01)
   
   rate_exp <- function(.time) 0.1
@@ -811,8 +771,8 @@ test_that("qtimecov works for all supported distributions", {
   init_luck <- 0.95
   
   expect_equal({
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005, max_time = time_change, return_luck = TRUE)
-    qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005, max_time = time_change)
+    qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)$tte
     
   },{
     new_luck <- luck_adj(prevsurv = 1 - pexp(q=time_change,rate_exp(1)),
@@ -832,8 +792,8 @@ test_that("qtimecov works for all supported distributions", {
   init_luck <- 0.95
   
   expect_equal({
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005, max_time = time_change, return_luck = TRUE)
-    qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005, max_time = time_change)
+    qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)$tte
     
   },{# Manually reproduce what qtimecov does from a$tte
     t <- 0
@@ -882,7 +842,7 @@ test_that("qtimecov works for all supported distributions", {
   
   #30x faster 
   expect_equal({
-    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 1)
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 1)$tte
     a
   },{
     time_vec <- 0:100
@@ -936,7 +896,7 @@ test_that("qtimecov works for all supported distributions", {
     luck <- 0.403
     time_cov <- 0.0002271
     t <- 0
-    qtimecov(luck = luck,a_fun = bs_rate_f,dist = "exp", dt = 1)
+    qtimecov(luck = luck,a_fun = bs_rate_f,dist = "exp", dt = 1)$tte
   }, tolerance = 0.01)
   
 })
@@ -955,7 +915,7 @@ test_that("qtimecov respects max_time bound", {
     a_fun = slow_fun,
     dist = "exp",
     max_time = 10
-  )
+  )$tte
   expect_lte(tte, 10)
 })
 
@@ -1035,3 +995,122 @@ test_that("qcondweibull and qcondweibullPH can give same results",
             qcond_weibullPH(p, shape, scale_PH, lower_bound = t0))
             }
           )
+
+
+
+
+# shared inputs -----------------------------------------------------------
+
+test_that("immutable mode: modify returns new object; originals unaffected", {
+  a <- shared_input(5, constrained = FALSE)
+  expect_s3_class(a, c("shared_input_val", "shared_input"))
+  
+  a2 <- a$modify(a$value() + 7)
+  expect_equal(a$value(), 5)
+  expect_equal(a2$value(), 12)
+  
+  # clone is a deep copy of the wrapper's current value
+  a3 <- a2$clone()
+  a2b <- a2$modify(99)
+  expect_equal(a2$value(), 12)     # old handle unchanged
+  expect_equal(a2b$value(), 99)
+  expect_equal(a3$value(), 12)     # clone still has 12
+})
+
+test_that("immutable mode: reset and fork work", {
+  a <- shared_input(10, constrained = FALSE)
+  
+  # reset goes back to initial and returns a fresh handle
+  a2 <- a$modify(11)
+  a3 <- a2$reset()
+  expect_equal(a2$value(), 11) # old handle unchanged
+  expect_equal(a3$value(), 10) # new handle reset to init
+  
+  # fork returns n independent deep clones (of the wrapper)
+  forks <- a$fork(3)
+  expect_type(forks, "list")
+  expect_length(forks, 3)
+  expect_true(all(vapply(forks, function(x) x$value(), numeric(1)) == 10))
+  
+  forks[[1]] <- forks[[1]]$modify(77)
+  expect_equal(forks[[1]]$value(), 77)
+  expect_equal(forks[[2]]$value(), 10) # independence across forks
+})
+
+test_that("shared mode: aliasing across handles and clone/reset break sharing", {
+  make_shared <- function(val) {
+    constrained <- TRUE  # parent flag consumed by shared_input()
+    shared_input(val)
+  }
+  
+  b1 <- make_shared(10)
+  expect_s3_class(b1, c("shared_input_env", "shared_input"))
+  
+  # alias: same underlying state
+  b2 <- b1
+  expect_equal(b1$value(), 10)
+  expect_equal(b2$value(), 10)
+  
+  # modify returns a fresh wrapper that *shares* the same state
+  b1 <- b1$modify(b1$value() + 1)
+  expect_equal(b1$value(), 11)
+  expect_equal(b2$value(), 11)  # reflects shared state
+  
+  # clone: new independent state
+  b3 <- b1$clone()
+  b1 <- b1$modify(99)
+  expect_equal(b1$value(), 99)
+  expect_equal(b2$value(), 99)  # still sharing with b1
+  expect_equal(b3$value(), 11)  # independent
+  
+  # reset: new independent state set to init
+  b4 <- b1$reset()
+  expect_equal(b4$value(), 10)  # initial value
+  b1 <- b1$modify(123)
+  expect_equal(b4$value(), 10)  # independent from b1/b2
+  
+  b5 <- b3$modify(b3$value() + 1)
+  
+  expect_equal(b5$value(), b3$value())  # same value
+})
+
+test_that("explicit constrained arg overrides parent lookup", {
+  constrained <- TRUE
+  x <- shared_input(1, constrained = FALSE)
+  expect_s3_class(x, "shared_input_val")
+  y <- shared_input(1, constrained = TRUE)
+  expect_s3_class(y, "shared_input_env")
+})
+
+test_that("parent lookup defaults to immutable unless TRUE", {
+  # no 'constrained' in parent -> immutable
+  x <- (function() shared_input(1))()
+  expect_s3_class(x, "shared_input_val")
+  
+  # constrained present but FALSE -> immutable
+  x <- (function() { constrained <- FALSE; shared_input(1) })()
+  expect_s3_class(x, "shared_input_val")
+  
+  # constrained present and TRUE -> shared
+  x <- (function() { constrained <- TRUE; shared_input(1) })()
+  expect_s3_class(x, "shared_input_env")
+})
+
+test_that("works with non-scalar values; note on reference types", {
+  # list value copies fine (but elements can be reference types)
+  a <- shared_input(list(x = 1L), constrained = FALSE)
+  a2 <- a$modify(list(x = 2L))
+  expect_equal(a$value(), list(x = 1L))
+  expect_equal(a2$value(), list(x = 2L))
+  
+  # environment as value: wrapper deep-copies the handle, but *not* the env itself
+  e <- new.env(parent = emptyenv()); e$k <- 1L
+  a <- shared_input(e, constrained = FALSE)
+  a2 <- a$clone()
+  e$k <- 99L
+  # both see 99 because the underlying env is shared by R semantics
+  expect_equal(a$value()$k, 99L)
+  expect_equal(a2$value()$k, 99L)
+})
+
+
